@@ -1,9 +1,8 @@
 package Mojolicious::Plugin::RoutesAuthDBI;
 use Mojo::Base 'Mojolicious::Plugin::Authentication';
-use Mojo::Loader qw(load_class);
+use Mojolicious::Plugin::RoutesAuthDBI::Util qw(load_class);
 use Mojo::Util qw(hmac_sha1_sum);
 use Hash::Merge qw( merge );
-use DBIx::POS::Sth;
 
 my $pkg = __PACKAGE__;
 
@@ -11,6 +10,7 @@ has [qw(app dbh conf)];
 
 has default => sub {
   my $self = shift;
+  require Mojolicious::Plugin::RoutesAuthDBI::Schema;
   {
   auth => {
     stash_key => $pkg,
@@ -28,87 +28,52 @@ has default => sub {
       shift->render(format=>'txt', text=>"You don`t have access on this route (url, action).\n");
     },
     import => [qw(load_user validate_user)],
-    pos => {
-      namespace => $pkg,
-      module => 'POS::Access',
-    },
   },
   admin => {
     namespace => $pkg,
     controller => 'Admin',
     prefix => lc($self->conf->{admin}{controller} || 'admin'),
     trust => hmac_sha1_sum('admin', $self->app->secrets->[0]),
-    pos => {
-      namespace => $pkg,
-      module => 'POS::Admin',
-    },
   },
   oauth => {
     namespace => $pkg,
     controller => 'OAuth2',
-    pos => {
-      namespace => $pkg,
-      module => 'POS::OAuth2',
-    },
     fail_auth_cb => sub {shift->render(format=>'txt', text=>"@_")},
   },
+  template => $Mojolicious::Plugin::RoutesAuthDBI::Schema::defaults,
 }};
 
 has merge_conf => sub {#hashref
   my $self = shift;
+  #~ @{ $self->conf }{qw(app plugin)} = ($self->app, $self);
   merge($self->conf, $self->default);
 };
 
 has access => sub {# object
   my $self = shift;
-  my $access = $self->merge_conf->{'access'};
-  my $class = $self->_class($access);
-  $class->import( @{$access->{import}});
-  bless $access, $class;
-  $access->{dbh} = $self->dbh;
-  my $pos = $access->{pos};
-  $access->{sth} = DBIx::POS::Sth->new(
-    $self->dbh,
-    $self->_class($pos)->new($pos->{template} ? (template=>$pos->{template}) : ())
-  );
-  $access->{app} = $self->app;
-  $access->{plugin} = $self;
-  return $access->init;
+  my $conf = $self->merge_conf->{'access'};
+  my $class = load_class($conf);
+  $class->import( @{ $conf->{import} });
+  $class->new(app=>$self->app, plugin=>$self,);
 };
 
 has admin => sub {# object
   my $self = shift;
-  my $admin = $self->merge_conf->{'admin'};
-  $admin->{module} ||= $admin->{controller};
-  my $class = $self->_class($admin);
-  bless $admin, $class;
-  $admin->{dbh} = $self->dbh;
-  my $pos = $admin->{pos};
-  $admin->{sth} = DBIx::POS::Sth->new(
-    $self->dbh,
-    $self->_class($pos)->new($pos->{template} ? (template=>$pos->{template}) : ()),
-  );
-  $admin->{plugin} = $self;
-  return $admin->init;
+  my $conf = $self->merge_conf->{'admin'};
+  #~ $conf->{module} ||= $conf->{controller};
+  load_class($conf)->init(%$conf, app=>$self->app, plugin=>$self,);
 };
 
 has oauth => sub {
   my $self = shift;
-  my $oauth = $self->merge_conf->{'oauth'};
-  my $class = $self->_class($oauth);
-  bless $oauth, $class;
-  $oauth->{dbh} = $self->dbh;
-  my $pos = $oauth->{pos};
-  $oauth->{sth} = DBIx::POS::Sth->new(
-    $self->dbh,
-    $self->_class($pos)->new($pos->{template} ? (template=>$pos->{template}) : ()),
-  );
-  $oauth->{app} = $self->app;
-  $oauth->{plugin} = $self;
-  #~ $oauth->{admin} = $self->admin;
-  return $oauth->init;
+  my $conf = $self->merge_conf->{'oauth'};
+  load_class($conf)->init(%$conf, app=>$self->app, plugin=>$self,);
 };
 
+has model => sub {
+  my $m = { map {$_ => load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::$_")->new} qw(Profiles Namespaces Routes Refs Controllers Actions Roles Logins) };
+  
+};
 
 sub register {
   my $self = shift;
@@ -120,6 +85,9 @@ sub register {
     if ref($self->dbh) eq 'CODE';
   die "Plugin must work with dbh, see SYNOPSIS" unless $self->dbh;
   
+  # init base model
+  load_class('DBIx::Mojo::Model')->singleton(dbh=>$self->dbh, template_vars=>$self->merge_conf->{template}, mt=>{tag_start=>'{%', tag_end=>'%}'});
+  
   my $access = $self->access;
   
   die "Plugin [Authentication] already loaded"
@@ -129,7 +97,7 @@ sub register {
   
   $self->app->routes->add_condition(access => sub {$self->cond_access(@_)});
   $access->apply_ns();
-  $access->apply_route($_) for @{ $access->db_routes };
+  $access->apply_route($_) for @{ $access->routes };
   
   if ($self->conf->{oauth}) {
     my $oauth = $self->oauth;
@@ -148,19 +116,6 @@ sub register {
 
 }
 
-sub _class {
-  my $self = shift;
-  my $conf = shift;
-  my $class  = join '::', $conf->{namespace}, $conf->{module} || $conf->{controller}
-    if $conf->{namespace};
-  $class ||= $conf->{module} || $conf->{controller};
-  
-  my $e; $e = load_class($class)# success undef
-    and die $e;
-  return $class;
-}
-
-# 
 sub cond_access {# add_condition
   my $self= shift;
   my ($route, $c, $captures, $args) = @_;
@@ -177,7 +132,7 @@ sub cond_access {# add_condition
     and return 1 # не проверяем доступ
     unless $args->{auth};
   
-  my $fail_auth_cb = $access->{fail_auth_cb};
+  my $fail_auth_cb = $conf->{access}{fail_auth_cb};
   
   # не авторизовался
   $self->deny_log($route, $args, $u)
@@ -185,9 +140,6 @@ sub cond_access {# add_condition
     and return undef
     unless $u;
   
-  #  получить все группы
-  $access->load_user_roles($u);
-  
   # допустить если {auth=>'only'}
   $app->log->debug(sprintf(qq[Access allow [%s] for {auth}='only'],
     $route->pattern->unparsed,
@@ -206,7 +158,7 @@ sub cond_access {# add_condition
     return 0x01;
   }
 
-  my $id2 = [$u->{id}, map($_->{id}, grep !$_->{disable},@{$u->{roles}})];
+  my $id2 = [$u->{id}, map($_->{id}, grep !$_->{disable},@{$u->roles})];
   my $id1 = [grep $_, @$args{qw(id route_id action_id controller_id namespace_id)}];
   
   # explicit acces to route
@@ -231,13 +183,14 @@ sub cond_access {# add_condition
     && return 1;
   
   # implicit access to non db routes
-  my $controller = $args->{controller} || ucfirst(lc($route->pattern->defaults->{controller}));
+  my $controller = $args->{controller} || $route->pattern->defaults->{controller} && ucfirst(lc($route->pattern->defaults->{controller}));
   my $namespace = $args->{namespace} || $route->pattern->defaults->{namespace};
   if ($controller && !$namespace) {
-    (load_class($_.'::'.$controller) or ($namespace = $_) and last) for @{ $app->routes->namespaces };
+    (load_class(namespace=>$_, controller=>$controller) and ($namespace = $_) and last) for @{ $app->routes->namespaces };
+    #~ warn "FOUND CONTROLLER[$controller] in NAMESPACE: $namespace";
   }
   
-  my $fail_access_cb = $access->{fail_access_cb};
+  my $fail_access_cb = $conf->{access}{fail_access_cb};
   
   $self->deny_log($route, $args, $u)
     and $c->$fail_access_cb()
@@ -320,7 +273,7 @@ sub deny_log {
   );
 }
 
-our $VERSION = '0.707';
+our $VERSION = '0.781';
 
 =pod
 
@@ -334,11 +287,11 @@ our $VERSION = '0.707';
 
 =head1 VERSION
 
-0.707
+0.781
 
 =head1 NAME
 
-Mojolicious::Plugin::RoutesAuthDBI - from DBI sql-tables does generate routes, make authentication and make restrict access (authorization) to request. Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> on which is based.
+Mojolicious::Plugin::RoutesAuthDBI - from DBI tables does generate routes, make authentication and make restrict access (authorization) to requests. Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> and OAuth2 by L<Mojolicious::Plugin::OAuth2>.
 
 =head1 DB DESIGN DIAGRAM
 
@@ -352,6 +305,7 @@ First of all you will see L<SVG|https://github.com/mche/Mojolicious-Plugin-Route
     access => {...},
     admin => {...},
     oauth => {...},
+    template => {...},
   );
 
 
@@ -361,7 +315,7 @@ One option C<dbh> is mandatory, all other - optional.
 
 =head3 dbh
 
-Handler DBI connection where are tables: controllers, actions, routes, logins, profiles, roles, refs.
+Handler DBI connection where are tables: controllers, actions, routes, logins, profiles, roles, refs and oauth.
 
   dbh => $app->dbh,
   # or
@@ -441,6 +395,9 @@ You might define your own controller by passing options:
 
 See L<Mojolicious::Plugin::RoutesAuthDBI::OAuth2> for detail options list.
 
+=head3 template
+
+Hashref variables for SQL templates of models dictionaries. Defaults is C<$Mojolicious::Plugin::RoutesAuthDBI::Schema::defaults>. See L<Mojolicious::Plugin::RoutesAuthDBI::Model::Base>.
 
 =head1 INSTALL
 
@@ -506,7 +463,6 @@ The callback will get parameters: $profile, $route, $c, $captures, $args (this c
 
 Returns access instance obiect. See L<Mojolicious::Plugin::RoutesAuthDBI::Access> methods.
 
-  $c->access->db_routes;
   if ($c->access->access_explicit([1,2,3], [1,2,3])) {
     # yes, accessible
   }
