@@ -1,24 +1,13 @@
 package Mojolicious::Plugin::RoutesAuthDBI::Admin;
 use Mojo::Base 'Mojolicious::Controller';
 
-
 my $pkg = __PACKAGE__;
-my ($dbh, $sth, $init_conf);
-has [qw(dbh sth plugin)];
+my ($Init);
+has [qw(plugin controller namespace prefix trust)];
 
 sub init {# from plugin! init Class vars
-  my $self = shift;
-  my %args = @_;
-
-  $self->dbh($self->{dbh} || $args{dbh});
-  $dbh = $self->dbh
-    or die "Нет DBI handler";
-  $self->sth($self->{sth} || $args{sth});
-  $sth = $self->sth
-    or die "Нет STH";
-  #~ $app = $self->app($self->{app} || $args{app});
-  $self->plugin($self->{plugin} || $args{plugin});
-  $init_conf = $self;
+  state $self = shift->SUPER::new(@_);
+  $Init = $self;
   return $self;
 }
 
@@ -48,7 +37,7 @@ TXT
 sub sign {
   my $c = shift;
   
-  $c->authenticate($c->vars('login','pass'))
+  $c->authenticate($c->vars(qw'login pass'))
     and $c->redirect_to("admin home")
     #~ and $c->render(format=>'txt', text=>__PACKAGE__ . "\n\nSuccessfull signed! ".$c->dumper( $c->auth_user))
     and return;
@@ -69,11 +58,11 @@ sub signout {
 sub users {
   my $c = shift;
   
-  my $p = $dbh->selectall_arrayref($sth->sth('profiles'), {Slice=>{}},);
+  my $p = $Init->plugin->model->{Profiles}->profiles;
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-Profiles
+Profiles(@{[scalar @$p]})
 ===
 
 @{[$c->dumper( $p)]}
@@ -83,10 +72,10 @@ TXT
 sub new_user {
   my $c = shift;
   
-  my ($login, $pass) = $c->vars('login', 'pass');
+  my ($login, $pass) = $c->vars(qw'login pass');
   
   my $r;
-  ($r = $dbh->selectrow_hashref($sth->sth('profile'), undef, (undef, $login)))
+  ($r = $Init->plugin->model->{Profiles}->get_profile(undef, $login))
     and $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -98,11 +87,11 @@ TXT
     and ($r->{not_new} = '!')
     and return $r;
   
-  $r = $dbh->selectrow_hashref($sth->sth('new login'), undef, ($login, $pass));
+  $r = $Init->plugin->model->{Logins}->new_login($login, $pass);
   
-  my $p =  $dbh->selectrow_hashref($sth->sth('new profile'), undef, ([$login],));
+  my $p = $Init->plugin->model->{Profiles}->new_profile([$login],);
   
-  $c->ref($p->{id} => $r->{id});
+  $Init->plugin->model->{Refs}->refer($p->{id} => $r->{id});
   
   @$p{qw(login pass)} = @$r{qw(login pass)};
   
@@ -123,30 +112,30 @@ sub trust_new_user {
   my $u = $c->new_user;
   
   # ROLE
-  my $rl = $dbh->selectrow_hashref($sth->sth('role'), undef, (undef, 'admin'));
-  $rl ||= $dbh->selectrow_hashref($sth->sth('new role'), undef, ('admin'));
+  my $rl = $Init->plugin->model->{Roles}->get_role(undef, 'admin')
+    || $Init->plugin->model->{Roles}->new_role('admin');
   
   # REF role->user
-  my $ru = $c->ref($rl->{id} => $u->{id});
+  my $ru = $Init->plugin->model->{Refs}->refer($rl->{id} => $u->{id});
   
   # CONTROLLER
-  my $cc = $dbh->selectrow_hashref($sth->sth('controller', where=>"where controller=? and (namespace=? or (?::varchar is null and namespace is null))"), undef, ($init_conf->{controller}, ($init_conf->{namespace}) x 2,));
-  $cc ||= $dbh->selectrow_hashref($sth->sth('new controller'), undef, ($init_conf->{controller}, 'admin actions'));
+  my $cc = $Init->plugin->model->{Controllers}->controller_ns($Init->{controller}, ($Init->{namespace}) x 2,)
+    || $Init->plugin->model->{Controllers}->new_controller($Init->{controller}, 'admin actions');
   
   #Namespace
-  my $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, (undef, $init_conf->{namespace},));
-  $ns ||= $dbh->selectrow_hashref($sth->sth('new namespace'), undef, ($init_conf->{namespace}, 'plugin ns!', undef, undef,));
+  my $ns = $Init->plugin->model->{Namespaces}->namespace(undef, $Init->{namespace},)
+    || $Init->plugin->model->{Namespaces}->new_namespace($Init->{namespace}, 'plugin ns!', undef, undef,);
   
   #ref namespace -> controller
-  my $nc = $c->ref($ns->{id}, $cc->{id});
+  my $nc = $Init->plugin->model->{Refs}->refer($ns->{id}, $cc->{id});
   
   #REF namespace->role
-  my $cr = $c->ref($ns->{id}, $rl->{id});
+  my $cr = $Init->plugin->model->{Refs}->refer($ns->{id}, $rl->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-Success sign up new trust-admin-user with whole access to namespace=[$init_conf->{namespace}]
+Success sign up new trust-admin-user with whole access to namespace=[$Init->{namespace}]
 ===
 
 USER:
@@ -167,7 +156,7 @@ TXT
 sub new_role {
 	my $c = shift;
 	my ($name) = $c->vars('name');
-	my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, (undef, $name));
+	my $r = $Init->plugin->model->{Roles}->get_role(undef, $name);
 	$c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -179,7 +168,7 @@ Role exists
 TXT
 		and return $c
 		if $r;
-	$r = $dbh->selectrow_hashref($sth->sth('new role'), undef, ($name));
+	$r = $Init->plugin->model->{Roles}->new_role($name);
 	
 	$c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -195,8 +184,8 @@ TXT
 
 sub user_roles {
   my $c = shift;
-  my ($user) = $c->vars('user') || $c->vars('login');
-  my $u =  $dbh->selectrow_hashref($sth->sth('profile'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
+  my ($user) = $c->vars(qw'user login');# || $c->vars('login');
+  my $u =  $Init->plugin->model->{Profiles}->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -208,7 +197,7 @@ TXT
     and return
     unless $u;
   
-  my $r = $dbh->selectall_arrayref($sth->sth('profile roles'), { Slice => {} }, ($u->{id}));
+  my $r = $u->roles;
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -228,12 +217,26 @@ TXT
   
 }
 
+sub roles {
+  my $c = shift;
+  my $r = $Init->plugin->model->{Roles}->roles;
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+ROLES(@{[scalar @$r]})
+===
+---
+@{[$c->dumper( $r)]}
+
+TXT
+}
+
 sub new_role_user {
   my $c = shift;
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
+  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -243,10 +246,10 @@ Can't create new role by only digits[$role] in name
 TXT
     and return
     unless $r && $role =~ /\w/;
-  $r ||= $dbh->selectrow_hashref($sth->sth('new role'), undef, ($role)) ;
+  $r ||= $Init->plugin->model->{Roles}->new_role($role);
   
   my ($user) = $c->vars('user');
-  my $u =  $dbh->selectrow_hashref($sth->sth('profile'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
+  my $u = $Init->plugin->model->{Profiles}->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -258,30 +261,68 @@ TXT
     and return
     unless $u;
   
-  my $ref = $c->refs($r->{id} => $u->{id});
-  $c->render(format=>'txt', text=><<TXT)
-$pkg
-
-Allready ref ROLE[$r->{name}] -> USER [@{[$c->dumper( $u) =~ s/\s+//gr]}]
-===
-
-@{[$c->dumper( $ref)]}
-TXT
-    and return
-    if $ref;
-  
-  $ref = $c->ref($r->{id} => $u->{id});
+  my $ref = $Init->plugin->model->{Refs}->refer($r->{id} => $u->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-Success create ref ROLE[$r->{name}] -> USER [@{[$c->dumper( $u) =~ s/\s+//gr]}]
+Success assign ROLE[$r->{name}] -> USER [@{[$c->dumper( $u) =~ s/\s+//gr]}]
 ===
 
 @{[$c->dumper( $ref)]}
 TXT
   
+}
+
+# доступ к контроллеру
+sub new_role_controller {
+  my $c = shift;
   
+  my ($role) = $c->vars('role');
+  # ROLE
+  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  $c->render(format=>'txt', text=><<TXT)
+$pkg
+
+Can't create new role by only digits[$role] in name
+===
+
+TXT
+    and return
+    unless $r && $role =~ /\w/;
+  $r ||= $Init->plugin->model->{Roles}->new_role($role);
+  
+  my ($ns, $controll) = $c->vars(qw'ns controll');
+  my $cntr = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns);
+  
+  $c->render(format=>'txt', text=><<TXT)
+$pkg
+
+No such controller [$ns::$controll]
+===
+
+TXT
+    and return
+    unless $cntr;
+  
+  my $ref = $Init->plugin->model->{Refs}->refer($cntr->{id} => $r->{id},);
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+Success assign access
+
+CONTROLLER 
+===
+@{[$c->dumper( $cntr) ]}
+
+ROLE
+===
+@{[$c->dumper( $r)]}
+
+REF
+===
+@{[$c->dumper( $ref)]}
+TXT
 }
 
 sub del_role_user {# удалить связь пользователя с ролью
@@ -289,7 +330,7 @@ sub del_role_user {# удалить связь пользователя с ро
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
+  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -301,7 +342,7 @@ TXT
     unless $r;
 
   my ($user) = $c->vars('user');
-  my $u =  $dbh->selectrow_hashref($sth->sth('profile'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
+  my $u = $Init->plugin->model->{Profiles}->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -313,7 +354,7 @@ TXT
     and return
     unless $u;
   
-  my $ref = $dbh->selectrow_hashref($sth->sth('del ref'), undef, (undef, $r->{id}, $u->{id}));
+  my $ref = $Init->plugin->model->{Refs}->del(undef, $r->{id}, $u->{id});
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -341,7 +382,7 @@ sub disable_role {
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $dbh->selectrow_hashref($sth->sth('dsbl/enbl role'), undef, ($a, $role =~ /\D/ ? (undef, $role) : ($role, undef,)));
+  my $r = $Init->plugin->model->{Roles}->dsbl_enbl($a, $role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -371,7 +412,7 @@ sub role_users {# все пользователи роли по запросу /
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
+  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -382,11 +423,11 @@ TXT
     and return
     unless $r;
   
-  my $u = $dbh->selectall_arrayref($sth->sth('role profiles'), { Slice => {} }, ($r->{id}));
+  my $u = $Init->plugin->model->{Roles}->profiles($r->{id});
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-All @{[scalar @$u]} profile/users by role [$r->{name}]
+Profile/users(@{[scalar @$u]}) by role [$r->{name}]
 ===
 
 @{[$c->dumper( $u)]}
@@ -398,7 +439,7 @@ sub role_routes {# все маршруты роли по запросу /myadmin
   
    my ($role) = $c->vars('role');
   # ROLE
-  my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
+  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -408,7 +449,7 @@ TXT
     and return
     unless $r;
   
-  my $t = $dbh->selectall_arrayref($sth->sth('role routes'), { Slice => {} }, ($r->{id}));
+  my $t = $Init->plugin->model->{Routes}->routes_ref($r->{id});
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -420,7 +461,7 @@ TXT
 
 sub controllers {
   my $c = shift;
-  my $list = $dbh->selectall_arrayref($sth->sth('controllers'), { Slice => {} }, );
+  my $list = $Init->plugin->model->{Controllers}->controllers;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -431,26 +472,27 @@ CONTROLLERS (@{[scalar @$list]})
 TXT
 }
 
-sub controller {# /controller/:ns/:controll
+sub controll {# /controller/:ns/:controll
   my $c = shift;
   my ($ns, $controll) = $c->vars(qw(ns controll));
-  my $list = $dbh->selectall_arrayref($sth->sth('controller', where=>"where (id=? or controller=?) and (namespace_id = ? or namespace = ? or (?::varchar is null and namespace is null))"), { Slice => {} }, ($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns));
+  my $r = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns);
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-CONTROLLER (@{[scalar @$list]})
+CONTROLLER
 ===
 
-@{[$c->dumper( $list)]}
+@{[$c->dumper( $r)]}
 TXT
 }
 
 sub new_controller {
   my $c = shift;
   #~ my $ns = $c->stash('ns') || $c->param('ns') ||  $c->stash('namespace') || $c->param('namespace');
-  my ($ns) = $c->vars('ns') || $c->vars('namespace');
-  my ($mod) = $c->vars('module');
-  my $cn = $dbh->selectrow_hashref($sth->sth('controller'), undef, ($mod, ($ns) x 2,));
+  my ($ns) = $c->vars(qw'ns namespace');# || ($c->vars('namespace'));
+  my ($mod) = $c->vars(qw'module controll');
+  my ($descr) = $c->vars(qw'descr');
+  my $cn = $Init->plugin->model->{Controllers}->controller_ns($mod, ($ns) x 2,);
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -462,11 +504,11 @@ TXT
   and return
   if $cn;
   my $n = $c->new_namespace($ns) if $ns;
-  $cn = $dbh->selectrow_hashref($sth->sth('new controller'), undef, ($mod, undef));
-  $c->ref($n->{id}, $cn->{id})
+  $cn = $Init->plugin->model->{Controllers}->new_controller($mod, $descr);
+  $Init->plugin->model->{Refs}->refer($n->{id}, $cn->{id})
     if $n;
   
-  $cn = $dbh->selectrow_hashref($sth->sth('controller'), undef, ($mod, ($ns) x 2,));
+  $cn = $Init->plugin->model->{Controllers}->controller_ns($mod, ($ns) x 2,);
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -481,7 +523,7 @@ TXT
 
 sub namespaces {
   my $c = shift;
-  my $list = $dbh->selectall_arrayref($sth->sth('namespaces'), { Slice => {} }, );
+  my $list = $Init->plugin->model->{Namespaces}->namespaces;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -498,7 +540,7 @@ sub new_namespace {
   my ($descr) = $_[0] ? (shift) :  $c->vars('descr');
   my ($app_ns) = $_[0] ? (shift) : $c->vars('app_ns');
   my ($interval_ts) = $_[0] ? (shift) : $c->vars('interval_ts');
-  my $n = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)));
+  my $n = $Init->plugin->model->{Namespaces}->namespace($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -509,7 +551,7 @@ Namespace already exists
 TXT
   and return $n
   if $n;
-  $n = $dbh->selectrow_hashref($sth->sth('new namespace'), undef, ($ns, $descr, $app_ns, $interval_ts));
+  $n = $Init->plugin->model->{Namespaces}->new_namespace($ns, $descr, $app_ns, $interval_ts);
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -524,9 +566,9 @@ TXT
 
 sub actions {
   my $c = shift;
-  my $list = $dbh->selectall_arrayref($sth->sth('actions'), { Slice => {} }, );
+  my $list = $Init->plugin->model->{Actions}->actions;
   map {
-    $_->{routes} = $dbh->selectall_arrayref($sth->sth('action routes', where=>"where action_id=?"), { Slice => {} }, ($_->{id}));
+    $_->{routes} = $Init->plugin->model->{Routes}->routes_action($_->{id});
   } @$list;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -540,12 +582,12 @@ TXT
 
 sub routes {
   my $c = shift;
-  my $list = $dbh->selectall_arrayref($sth->sth('apply routes'), { Slice => {} }, );
+  my $list = $Init->plugin->model->{Routes}->routes;
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-ROUTES list (@{[scalar @$list]})
+ROUTES (@{[scalar @$list]})
 ===
 
 @{[$c->dumper( $list )]}
@@ -554,7 +596,7 @@ TXT
 
 sub new_route_ns {# показать список мест-имен
   my $c = shift;
-  my $list = $dbh->selectall_arrayref($sth->sth('namespaces'), { Slice => {} }, );
+  my $list = $Init->plugin->model->{Namespaces}->namespaces;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -569,11 +611,12 @@ TXT
 }
 
 sub new_route_c {# показать список контроллеров
+  no warnings;
   my $c = shift;
   my ($ns) = $c->vars('ns');
-  $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
+  $ns = $Init->plugin->model->{Namespaces}->namespace($ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
     || {namespace => $ns};
-  my $list = $dbh->selectall_arrayref($sth->sth('controllers', where=>"where n.id=? or (?::int is null and n.id is null)"), { Slice => {} }, ($ns->{id}, $ns->{id}));
+  my $list = $Init->plugin->model->{Controllers}->controllers_ns_id($ns->{id}, $ns->{id});
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -589,17 +632,18 @@ TXT
 }
 
 sub new_route_a {# показать список действий
+  no warnings;
   my $c = shift;
-  my ($ns, $controll) = $c->vars('ns', 'controll');
+  my ($ns, $controll) = $c->vars(qw'ns controll');
   
-  $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
+  $ns = $Init->plugin->model->{Namespaces}->namespace($ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
     || {namespace => $ns};
   
-  $controll = $dbh->selectrow_hashref($sth->sth('controllers', where=>"where (n.id=? or (?::varchar is null and n.id is null)) and (c.id=? or c.controller=?)"), undef, ($ns->{id}, $ns->{id}, $controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), ))
+  $controll = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), $ns->{id}, $ns->{namespace}, $ns->{namespace},)
     || {controller=>$controll};
   
-  my $list = $dbh->selectall_arrayref($sth->sth('actions', where=>"where controller_id=?"), { Slice => {} }, ($controll->{id}));
-  my $list2 = $dbh->selectall_arrayref($sth->sth('actions', where=>"where controller_id is null"), { Slice => {} }, ());
+  my $list = $Init->plugin->model->{Actions}->actions_controller($controll->{id});
+  my $list2 = $Init->plugin->model->{Actions}->actions_controller_null();
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -622,16 +666,16 @@ TXT
 my @route_cols = qw(request name descr auth disable interval_ts);
 sub new_route {# показать маршруты к действию
   my $c = shift;
-  my ($ns, $controll, $act) = $c->vars('ns', 'controll', 'act');
+  my ($ns, $controll, $act) = $c->vars(qw'ns controll act');
   
-  $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
+  $ns = $Init->plugin->model->{Namespaces}->namespace($ns&& $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
     || {namespace => $ns};
   
-  $controll = $dbh->selectrow_hashref($sth->sth('controllers', where=>"where (n.id=? or (?::varchar is null and n.id is null)) and (c.id=? or c.controller=?)"), undef, ($ns->{id}, $ns->{id}, $controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), ))
+  $controll = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), $ns->{id}, $ns->{namespace}, $ns->{namespace},)
     || {controller=>$controll};
   
-  $act = $dbh->selectrow_hashref($sth->sth('actions', where=>"where controller_id=? and (a.id = ? or a.action = ? )"), undef, ($controll->{id}, $act =~ /\D/ ? (undef, $act) : ($act, undef,),))
-    || $dbh->selectrow_hashref($sth->sth('actions', where=>"where controller_id is null and (a.id = ? or a.action = ? )"), undef, ($act =~ /\D/ ? (undef, $act) : ($act, undef,),))
+  $act = $Init->plugin->model->{Actions}->action_controller($controll->{id}, $act =~ /\D/ ? (undef, $act) : ($act, undef,),)
+    || $Init->plugin->model->{Actions}->action_controller_null($act =~ /\D/ ? (undef, $act) : ($act, undef,),)
     || {action => $act};
   
   # Проверка на похожий $request ?? TODO
@@ -672,11 +716,10 @@ TXT
   
   
   # маршруты действия
-  my $list = $act->{id} ? $dbh->selectall_arrayref($sth->sth('action routes', where=>'where action_id=?'), { Slice => {} }, ($act->{id}))
+  my $list = $act->{id} ? $Init->plugin->model->{Routes}->routes_action($act->{id})
     : [];
   # свободные маршруты
-  my $list2 = $act->{id} ? $dbh->selectall_arrayref($sth->sth('action routes', where=>'where action_id is null'), { Slice => {} }, ())
-    : [];
+  my $list2 = $Init->plugin->model->{Routes}->routes_action_null;
   
   no warnings;
   $c->render(format=>'txt', text=><<TXT);
@@ -713,20 +756,21 @@ TXT
 sub route_save {
   my $c = shift;
   my ($ns, $controll, $act, $route) = @_;
-  local $dbh->{AutoCommit} = 0;
-  $ns = $dbh->selectrow_hashref($sth->sth('new namespace'), undef, (@$ns{qw(namespace descr app_ns interval_ts)}))
+  local $Init->plugin->dbh->{AutoCommit} = 0;
+  $ns = $Init->plugin->model->{Namespaces}->new_namespace(@$ns{qw(namespace descr app_ns interval_ts)})
     if $ns->{namespace} && ! $ns->{id};
-  $controll = $dbh->selectrow_hashref($sth->sth('new controller'), undef, (@$controll{qw(controller descr)}))
+  $controll = $Init->plugin->model->{Controllers}->new_controller(@$controll{qw(controller descr)})
     unless $controll->{id};
-  $act = $dbh->selectrow_hashref($sth->sth('new action'), undef, (@$act{qw(action callback descr)}))
+  $act = $Init->plugin->model->{Actions}->new_action(@$act{qw(action callback descr)})
     unless $act->{id};
     
-  $route = $dbh->selectrow_hashref($sth->sth('new route'), undef, (@$route{@route_cols}))
+  $route = $Init->plugin->model->{Routes}->new_route(@$route{@route_cols})
     unless $route->{id};
   my $ref = [map {
-    $c->ref($$_[0]{id}, $$_[1]{id},) if $$_[0]{id} && $$_[1]{id};
+    $Init->plugin->model->{Refs}->refer($$_[0]{id}, $$_[1]{id},)
+      if $$_[0]{id} && $$_[1]{id};
   } ([$ns, $controll], [$controll, $act], [$route, $act],)];
-  $dbh->commit;
+  $Init->plugin->dbh->commit;
   return ($ns, $controll, $act, $route, $ref);
 
 }
@@ -741,21 +785,19 @@ sub vars {# получить из stash || param
   } @_;
 }
 
-sub ref {# get or save
-  my $c = shift;
-  my ($id1, $id2) = @_;
-    #~ $c->app->log->debug($c->dumper(\@_));
-  $dbh->selectrow_hashref($sth->sth('ref'), undef, ($id1, $id2,))
-    || $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($id1, $id2,));
-}
+#~ sub ref {# get or save
+  #~ my $c = shift;
+  #~ my ($id1, $id2) = @_;
+  #~ $Init->plugin->model->{Refs}->refer($id1, $id2,);
+#~ }
 
 
 
 my @self_routes_cols = qw(request action name auth descr);
 sub self_routes {# from plugin!
   my $c = shift;
-  my $prefix = $init_conf->{prefix};
-  my $trust = $init_conf->{trust};
+  my $prefix = $Init->prefix;
+  my $trust = $Init->trust;
 
   my $t = <<TABLE;
 /$prefix	index	admin home	1	Main page
@@ -766,7 +808,7 @@ sub self_routes {# from plugin!
 /$prefix/namespace/new/:ns/:descr/:app_ns	new_namespace	$prefix new_namespace	1	Add new ns
 /$prefix/controllers	controllers	$prefix controllers	1	Controllers list
 /$prefix/controller/new/:ns/:module	new_controller	$prefix new_controller	1	Add new controller by :ns and :module
-/$prefix/controller/:ns/:controll	controller	$prefix controller	1	View a controller (ID and name for NS and controller)
+/$prefix/controller/:ns/:controll	controll	$prefix controller	1	View a controller (ID and name for NS and controller)
 /$prefix/actions	actions	$prefix actions	1	Actions list
 #
 # Роли и доступ
@@ -777,7 +819,9 @@ sub self_routes {# from plugin!
 /$prefix/role/enbl/:role	enable_role	$prefix enable role->user	1	Enable :role by role.id|role.name.
 /$prefix/roles	roles	$prefix view roles	1	View roles table
 /$prefix/roles/:user	user_roles	$prefix roles of user	1	View roles of :user by profile.id|login
-/$prefix/role/:role/:user	new_role_user	$prefix create ref role->profile	1	Assign :user to :role by profile.id|login and role.id|role.name.
+/$prefix/role/user/:role/:user	new_role_user	$prefix create ref role->profile	1	Assign :user to :role by profile.id|login and role.id|role.name.
+/$prefix/role/namespace/:role/:ns	new_role_namespace	$prefix create ref namespace->role	1	Assign :role to :ns by role.id|role.name and namespace.id|namespace.name.
+/$prefix/role/controller/:role/:ns/:controll	new_role_controller	$prefix create ref controller->role	1	Assign :role to :controll by role.id|role.name and controller.id|controller.name+namespace(id|name).
 #
 # Последовательный ввод нового маршрута
 #
@@ -814,8 +858,8 @@ TABLE
   for my $line (grep /\S+/, split /\n/, $t) {
     my $r = {};
     @$r{@self_routes_cols} = map($_ eq '' ? undef : $_, split /\t/, $line);
-    $r->{namespace} = $init_conf->{namespace};
-    $r->{controller} = $init_conf->{controller};
+    $r->{namespace} = $Init->{namespace};
+    $r->{controller} = $Init->{controller};
     push @r, $r;
   }
   
@@ -865,9 +909,6 @@ Both above options determining the module controller for web actions on tables r
 
 =item * B<trust> is a url subprefix for trust admin urls of this module. See defaults below.
 
-=item * B<pos> - hashref
-
-SQL-dictionary for DBI statements. See L<Mojolicious::Plugin::RoutesAuthDBI::POS::Admin>.
 
 =back
 
@@ -875,13 +916,9 @@ SQL-dictionary for DBI statements. See L<Mojolicious::Plugin::RoutesAuthDBI::POS
 
   admin = > {
     namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
-    module => 'Admin',
+    controller => 'Admin',
     prefix => 'admin', # lc(<module>)
     trust => hmac_sha1_sum('admin', $app->secrets->[0]),
-    pos => {
-      namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
-      module => 'POS::Admin',
-    }, 
   },
     
   admin = {}, # empty hashref sets defaults above
